/**
 * 存在该配置文件，主要是因为方便子项目进行重写一些 webpack.config 里面的配置
 */
const path = require('path')
const fs = require('fs')

const appDirectory = fs.realpathSync(process.cwd())
const resolveApp = relativePath => path.resolve(appDirectory, relativePath)

// 加载项目自定义配置
const loadConfig = () => {
  const configPath = resolveApp('./config.js')
  let config = {}

  if (fs.existsSync(configPath)) {
    config = require(configPath)
  }

  return config
}

const defaultConfig = {
  // 代理配置
  proxy: {},
  // 入口目录
  inputPath: '',
  // 构建目录
  outputPath: 'dist',
  // 文件引用路径别名
  alias: {},
  // 外部引用
  externals: {},
  // rem 转换基准
  basePixel: 16,
  // 公共模块
  chunks: {},
  // 编译地图开关
  sourceMap: false,
  // js 选择性注入判断函数
  injectCheck: (html, js) => {
    return html === js
  }
}

const projectConfig = loadConfig()
const CONFIG = { ...defaultConfig, ...projectConfig }

CONFIG.outputPath = resolveApp(CONFIG.outputPath)

module.exports = CONFIG
