/**
 * 2018-11-02 11:50:16
 * 生成 html plugins 和 js 的 entry 配置
 *
 * pug 模板和 js 入口 如何匹配的方法，可以具体项目里面传进来。
 */
const path = require('path')
const glob = require('glob')
const HtmlWebpackPlugin = require('html-webpack-plugin')
const paths = require('./paths')
const BUILD_CONFIG = require('./build-config')

// 根据匹配规则输出正确的文件路径
const getEntries = (pattern, hotReload) => {
  let fileList = glob.sync(`${paths['appPath']}/${pattern}`)
  return fileList.reduce((previous, current) => {
    // 过滤掉代码目录前的路径，只保留文件相对于项目根目录的文件
    // eg: xxxx/src/demo/index.js => demo/index.js
    let filePath = path.parse(current.replace(`${paths['appSrc']}/`, ''))

    // 文件生成后的路径和文件名
    let withoutSuffix = path.join(filePath.dir, filePath.name)
    if (hotReload) {
      // 多页面，需要对每个入口添加 热更新的配置
      previous[withoutSuffix] = [current, require.resolve('react-dev-utils/webpackHotDevClient')]
    } else {
      previous[withoutSuffix] = current
    }
    return previous
  }, {})
}

/**
 * 获取生成 html 的插件和 多页面的 js 入口
 *
 * @param {boolean} isDev 是否为开发环境 (开发环境，则添加 Hot Reload 功能)
 */
const getEntry = isDev => {
  const inputPath = BUILD_CONFIG.inputPath || ''
  const jsRegx = `src/${inputPath}/**/*.jsx`
  const htmlRegx = `src/${inputPath}/**/*.pug`
  const jsEntries = getEntries(jsRegx, isDev)
  const htmlEntries = getEntries(htmlRegx)

  let htmlPlugins = []
  for (let htmlEntry in htmlEntries) {
    const config = {
      filename: htmlEntry + '.html',
      template: htmlEntries[htmlEntry],
      chunks: [],
      inject: true,
      hash: true,
      cache: true,
      chunksSortMode: 'manual',
      minify: {
        removeComments: true,
        collapseWhitespace: false
      }
    }

    // 注入公共库
    for (key in BUILD_CONFIG.chunks) {
      config.chunks.push(key)
    }

    // 遍历判断注入
    for (let jsEntry in jsEntries) {
      if (BUILD_CONFIG.injectCheck(htmlEntry, jsEntry)) {
        config.chunks.push(jsEntry)
      }
    }
    htmlPlugins.push(new HtmlWebpackPlugin(config))
  }

  return {
    html: htmlPlugins,
    js: {
      ...BUILD_CONFIG.chunks,
      ...jsEntries
    }
  }
}

module.exports = getEntry
